#pragma semicolon 1;
#include <sdktools>
#define UPD_LIBFUNC
#define CONVAR_PREFIX "sm_namechangeinfo"
#define DEFAULT_UPDATE_SETTING "2"
#include <ddhoward_updater>
#pragma newdecls required;

#define CHAT_PREFIX "[NameChange]"

ConVar hcvar_destination;
ConVar hcvar_time;
ConVar hcvar_count;

float ChangeTime[MAXPLAYERS+1];
int ChangeCount[MAXPLAYERS+1];
int LastNameChanger = -1;

public Plugin myinfo = {
	name = "Target Name Changers (namechangeinfo.smx)",
	author = "Derek D. Howard",
	description = "Displays the STEAM ID, userID, and IP address of players who change their name, and facilitates easy targeting of said players.",
	version = "18.0114.0",
	url = "https://forums.alliedmods.net/showthread.php?t=219956"
};

public void OnPluginStart() {
	
	hcvar_destination = CreateConVar("sm_namechangeinfo_destination", "1", "(0/1/2/3) Where should the information be shown? 0 = disabled, 1 = Console, 2 = Chat, 3 = Both");
	hcvar_time = CreateConVar("sm_namechangeinfo_time", "60.0", "For how many seconds since a client's last name change should be be considered a name changer?");
	hcvar_count = CreateConVar("sm_namechangeinfo_count", "3", "Number of times a client can change names before being included in @freqnamechangers");

	HookEvent("player_changename", OnNameChange);
	
	AddMultiTargetFilter("@recentnamechangers", TargetRecentChangers, "players who have changed their names recently", false);
	AddMultiTargetFilter("@namechanger", TargetLastNameChanger, "the player who last changed his/her name", false);
	AddMultiTargetFilter("@namechangers", TargetFreqChangers, "players who are changing names frequently", false);
}

public void OnPluginEnd() {
	RemoveMultiTargetFilter("@recentnamechangers", TargetRecentChangers);
	RemoveMultiTargetFilter("@namechanger", TargetLastNameChanger);
	RemoveMultiTargetFilter("@namechangers", TargetFreqChangers);
}

public void OnNameChange(Event event, const char[] name, bool dontBroadcast) {
	int userID = event.GetInt("userid");
	int namechanger = GetClientOfUserId(userID);
	if (namechanger <= 0 || namechanger > MaxClients || !IsClientInGame(namechanger) || IsFakeClient(namechanger)) {
		return;
	}
	LastNameChanger = namechanger;

	if (ChangeTime[namechanger] > GetEngineTime() - hcvar_time.FloatValue) {
		ChangeCount[namechanger]++;
	} else { ChangeCount[namechanger] = 1; }

	ChangeTime[namechanger] = GetEngineTime();
	
	if (hcvar_destination.IntValue <= 0 || hcvar_destination.IntValue > 3) {
		return;
	}
	char steamID[22]; char ipAddress[16]; char oldName[MAX_NAME_LENGTH]; char newName[MAX_NAME_LENGTH]; char output[512];
	GetEventString(event, "oldname", oldName, sizeof(oldName));
	GetEventString(event, "newname", newName, sizeof(newName));
	if (!GetClientAuthId(namechanger, AuthId_Engine, steamID, sizeof(steamID))) {
		steamID = "[UNKNOWN]";
	}
	if (!GetClientIP(namechanger, ipAddress, sizeof(ipAddress), true)) {
		ipAddress = "[UNKNOWN]";
	}
	Format(output, sizeof(output), "%s \"%s\" changed name to \"%s\". UserID: #%i, SteamID: %s", CHAT_PREFIX, oldName, newName, userID, steamID);
	for (int client = 1; client <= MaxClients; client++) {
		if (IsClientInGame(client) && !IsFakeClient(client) && CheckCommandAccess(client, "sm_namechangeinfo", 0, true)) {
			if (CheckCommandAccess(client, "sm_namechangeinfo_seeip", ADMFLAG_BAN, true)) {
				if (hcvar_destination.IntValue & 1) {
					PrintToConsole(client, "%s, IP: %s", output, ipAddress);
				}
				if (hcvar_destination.IntValue & 2) {
					PrintToChat(client, "%s, IP: %s", output, ipAddress);
				}
			}
			else {
				if (hcvar_destination.IntValue & 1) {
					PrintToConsole(client, output);
				}
				if (hcvar_destination.IntValue & 2) {
					PrintToChat(client, output);
				}
			}
		}
	}
}

public bool TargetRecentChangers(const char[] pattern, Handle clients) {
	float targettime = GetEngineTime() - hcvar_time.FloatValue;
	for (int client = 1; client <= MaxClients; client++) {
		if (IsClientInGame(client) && ChangeTime[client] > targettime) {
			PushArrayCell(clients, client);
		}
	}
	return true;
}

public bool TargetLastNameChanger(const char[] pattern, Handle clients) {
	if (LastNameChanger > 0 && LastNameChanger <= MaxClients && IsClientInGame(LastNameChanger)) {
		PushArrayCell(clients, LastNameChanger);
	}
	return true;
}

public bool TargetFreqChangers(const char[] pattern, Handle clients) {
	for (int client = 1; client <= MaxClients; client++) {
		if (IsClientInGame(client)
		&& ChangeCount[client] >= hcvar_count.IntValue
		&& ChangeTime[client] > GetEngineTime() - hcvar_time.FloatValue) {
			PushArrayCell(clients, client);
		}
	}
	return true;
}

public void OnClientDisconnect(int client) {
	ChangeTime[client] = 0.0;
	ChangeCount[client] = 0;
	if (LastNameChanger == client) {
		LastNameChanger = -1;
	}
}